package com.sxtvs.open.api.clue.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sxtvs.open.api.clue.dto.ClueDTO;
import com.sxtvs.open.api.clue.dto.VerifyDTO;
import com.sxtvs.open.api.clue.dto.WxReply;
import com.sxtvs.open.api.clue.entity.*;
import com.sxtvs.open.api.clue.mapper.ClueMapper;
import com.sxtvs.open.api.clue.service.IClueService;
import com.sxtvs.open.api.h5template.service.impl.OSSService;
import com.sxtvs.open.core.advice.BizException;
import com.sxtvs.open.core.sls.AliyunLogger;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.tuple.ImmutablePair;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.http.util.TextUtils;
import org.apache.logging.log4j.util.Strings;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2023-03-20
 */
@Service
public class ClueServiceImpl extends ServiceImpl<ClueMapper, Clue> implements IClueService {

    @Resource
    private ClueReplyServiceImpl clueReplyService;

    @Resource
    private ClueInfoServiceImpl clueInfoService;

    @Resource
    private ObjectMapper objectMapper;

    @Resource
    private AliyunLogger logger;

    @Resource
    private OSSService ossService;

    public IPage<Clue> cluePage(ClueDTO clueDTO){

        LambdaQueryWrapper<Clue> wrapper = Wrappers.lambdaQuery();

        IPage<Clue> clueIPage = baseMapper.selectPage(new Page<>(clueDTO.getPage(), clueDTO.getSize()),
                wrapper.ge(clueDTO.getStart() != null, Clue::getAddTime, clueDTO.getStart())
                .lt(clueDTO.getEnd() != null, Clue::getAddTime, clueDTO.getEnd())
                .like(!TextUtils.isEmpty(clueDTO.getTitle()), Clue::getTitle, "%" + clueDTO.getTitle() + "%")
                .eq(Clue::getValidity, (byte) 1)
                .orderByDesc(Clue::getAddTime)
        );

        if(clueIPage.getTotal() > 0){
            Map<String, Pair<String, Integer>> collect = clueInfoService.lambdaQuery().in(
                    ClueInfo::getRelaId, clueIPage.getRecords().stream().map(Clue::getId).collect(Collectors.toList())
            ).list().stream().collect(Collectors.groupingBy(ClueInfo::getRelaId, Collectors.collectingAndThen(Collectors.toList(),
                    data -> new ImmutablePair<>(Strings.join(data.stream().map(ClueInfo::getType).distinct().collect(Collectors.toList()), ','), data.size())
            )));
            clueIPage.setRecords(clueIPage.getRecords().stream().peek(v -> {
                v.setClueInfoCount(collect.containsKey(v.getId()) ? collect.get(v.getId()).getRight() : 1);
                v.setClueInfoType(collect.containsKey(v.getId()) ? collect.get(v.getId()).getLeft() : "0");
            }).collect(Collectors.toList()));
        }

        return clueIPage;
    }

    public void store(Clue clue){
        clue.setId(null);
        clue.setAddTime(LocalDateTime.now());
        clue.setCreateTime(LocalDateTime.now());
        clue.setModifyTime(LocalDateTime.now());
        save(clue);
    }

    public void verify(VerifyDTO verifyDTO){
        Clue clue = getById(verifyDTO.getId());
        if(clue.getStatus() != 0){
            throw new BizException(40006, "该条数据已审核");
        }
        clue.setStatus(verifyDTO.getStatus());
        updateById(clue);
    }

    @Transactional
    public void reply(ClueReply clueReply){
        Clue clue = getById(clueReply.getClueId());
        clue.setReplyFlag(1);
        clue.setReplyTime(LocalDateTime.now());
        updateById(clue);
        clueReply.setId(null);
        clueReply.setReplyTime(LocalDateTime.now());
        clueReply.setModifyTime(LocalDateTime.now());

        //公众号
        if (clue.getSource() == 2){
            WxReply wxReply = new WxReply();
            wxReply.setMsgtype("text");
            WxReply.TextDTO textDTO = new WxReply.TextDTO();
            textDTO.setContent(clueReply.getContent());
            wxReply.setText(textDTO);
            wxReply.setTouser(clue.getAuthor());
            try {
                HttpResponse resp = HttpRequest.post("https://api.weixin.qq.com/cgi-bin/message/custom/send?access_token=" + WxTokenCache.getInstance().getAccessToken())
                        .body(objectMapper.writeValueAsString(wxReply)).contentType("application/json;charset=UTF-8").execute();
                System.out.println(resp.body());
                logger.info("wx reply response ", resp.body());
            } catch (JsonProcessingException e) {
                logger.error("wx reply error ", e);
            }

        }

        clueReplyService.save(clueReply);
    }

    @Transactional
    public void delete(Long id){
        Clue clue = getById(id);
        if(null == clue){
            throw new BizException(40006, "不存在的信息");
        }
        clue.setValidity((byte) 0);
        updateById(clue);
        clueReplyService.updateBatchById(
                clueReplyService.lambdaQuery().eq(ClueReply::getClueId, clue.getId()).list().stream()
                        .peek(data -> data.setStatus(-1)).collect(Collectors.toList())
        );
    }

    public Info info(String id){
        Clue clue = getById(id);
        Info info = new Info();
        BeanUtil.copyProperties(clue, info);
        info.setClueReplies(clueReplyService.lambdaQuery().eq(ClueReply::getClueId,clue.getId()).list());
        List<ClueInfo> list = clueInfoService.lambdaQuery().eq(ClueInfo::getRelaId, clue.getId()).list();
        if(list.size() == 0){
            ClueInfo clueInfo = new ClueInfo();
            clueInfo.setRelaId(clue.getId());
            clueInfo.setAddUser(clue.getAuthor());
            clueInfo.setMaterialUrl(clue.getContent());
            clueInfo.setAddTime(clue.getAddTime());
            clueInfo.setType(0);
            list.add(clueInfo);
        }
        info.setClueInfos(list);
        return info;
    }

    public void wxCreate(WxMessage wxMessage){
        LocalDateTime addTime = LocalDateTime.ofEpochSecond(wxMessage.getCreateTime(), 0, ZoneOffset.ofHours(8));

        Clue last = lambdaQuery().eq(Clue::getAuthor, wxMessage.getFromUserName()).gt(Clue::getAddTime, addTime.minusHours(24))
                .orderByDesc(Clue::getId).last("limit 1").one();
        if(null != last && last.getReplyFlag() == 0){   //24h内有消息但未回复
            List<ClueInfo> infos = clueInfoService.lambdaQuery().eq(ClueInfo::getRelaId, last.getId()).list();

            if(infos.size() == 0){
                ClueInfo info = new ClueInfo();
                info.setAddUser(last.getAuthor());
                info.setAddTime(last.getAddTime());
                info.setRelaId(last.getId());
                info.setMaterialUrl(last.getContent());
                info.setType(0);
                clueInfoService.save(info);
            }
            ClueInfo info1 = new ClueInfo();
            info1.setAddUser(wxMessage.getFromUserName());
            info1.setAddTime(addTime);
            info1.setRelaId(last.getId());
            info1.setType(getMediaType(wxMessage.getMsgType()));
            if(wxMessage.getMsgType().equals("text")){
                info1.setMaterialUrl(wxMessage.getContent());
            }else {
                String url = ossService.urlPut(wxMessage.getPicUrl(), "jpg");
                info1.setMaterialUrl(wxMessage.getPicUrl());
                info1.setLocalUrl(url);
                info1.setWechatMaterialId(wxMessage.getMediaId());
            }

            clueInfoService.save(info1);

        }else if(wxMessage.getMsgType().equals("text")){
            Clue clue = new Clue();
            clue.setSource(2);
            clue.setSiteId(3L);
            clue.setAscription(3);
            clue.setTitle(wxMessage.getContent());
            clue.setSummary(wxMessage.getContent());
            clue.setContent(wxMessage.getContent());
            clue.setAuthor(wxMessage.getFromUserName());
            clue.setAddTime(addTime);
            clue.setCreateTime(addTime);
            clue.setModifyTime(addTime);
            clue.setOriginTime(addTime);

            save(clue);
        }

    }

    private Integer getMediaType(String type){
        switch (type){
            case "text":
                return 0;
            case "image":
                return 1;
            case "video":
                return 2;
        }
        return 0;
    }

}
