package com.sxtvs.open.api.h5template.service.impl;

import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sxtvs.open.api.h5template.dto.PageRequestDTO;
import com.sxtvs.open.api.h5template.entity.H5OperateFormData;
import com.sxtvs.open.api.h5template.mapper.H5OperateFormDataMapper;
import com.sxtvs.open.api.h5template.service.IH5OperateFormDataService;
import com.sxtvs.open.core.advice.BizException;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2023-04-04
 */
@Service
public class H5OperateFormDataServiceImpl extends ServiceImpl<H5OperateFormDataMapper, H5OperateFormData> implements IH5OperateFormDataService {

    public void store(Map<String, Object> map){
        List<H5OperateFormData> h5OperateFormDataList = new ArrayList<>();
        if(!map.containsKey("operateId")){
            throw new BizException("缺少活动ID");
        }
        String operateId = (String) map.get("operateId");
        String formId = (String) map.get("formId");

        map.forEach((k, v) -> {
            H5OperateFormData h5OperateFormData = new H5OperateFormData();
            h5OperateFormData.setOperateId(operateId);
            h5OperateFormData.setCreateTime(LocalDateTime.now());
            h5OperateFormData.setUpdateTime(LocalDateTime.now());
            h5OperateFormData.setColumnKey(k);
            h5OperateFormData.setColumnValue(String.valueOf(v));
            h5OperateFormData.setFormId(formId);
            h5OperateFormDataList.add(h5OperateFormData);
        });

        if(map.containsKey("dataId")){
            Long dataId = (Long) map.get("dataId");
            h5OperateFormDataList.forEach(data -> update(
                    new LambdaUpdateWrapper<H5OperateFormData>().set(H5OperateFormData::getColumnValue, data.getColumnValue())
                            .eq(H5OperateFormData::getDataId, dataId).eq(H5OperateFormData::getColumnKey, data.getColumnKey())
            ));

        }else{
            String snowflakeNextId = String.valueOf(IdUtil.getSnowflakeNextId());
            saveBatch(h5OperateFormDataList.stream().peek(data -> data.setDataId(snowflakeNextId)).collect(Collectors.toList()));
        }
    }

    public Page<Map<String, Object>> pageList(PageRequestDTO pageRequestDTO){
        IPage<H5OperateFormData> page = baseMapper.selectPage(new Page<>(pageRequestDTO.getPage(), pageRequestDTO.getSize()),
                new LambdaQueryWrapper<H5OperateFormData>().select(H5OperateFormData::getDataId).groupBy(H5OperateFormData::getDataId)
                .orderByDesc(H5OperateFormData::getDataId)
        );

        Page<Map<String, Object>> rs = new Page<>(pageRequestDTO.getPage(), pageRequestDTO.getSize(), page.getTotal());
        rs.setPages(page.getPages());
        List<String> ids = page.getRecords().stream().map(H5OperateFormData::getDataId).collect(Collectors.toList());
        List<Map<String, Object>> ls = new ArrayList<>();
        lambdaQuery().in(H5OperateFormData::getDataId, ids).list().stream().collect(Collectors.groupingBy(H5OperateFormData::getDataId))
        .forEach((k, v) -> {
            Map<String, Object> map = new HashMap<>();
            map.put("dataId", k);
            v.forEach(h5 -> {
                map.put(h5.getColumnKey(), h5.getColumnValue());
            });
            ls.add(map);
        });

        rs.setRecords(ls);
        return rs;
    }
}
