package com.sxtvs.open.api.h5template.service.impl;

import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sxtvs.open.api.h5template.entity.H5Operate;
import com.sxtvs.open.api.h5template.entity.H5OperateForm;
import com.sxtvs.open.api.h5template.entity.H5OperateFormData;
import com.sxtvs.open.api.h5template.mapper.H5OperateFormMapper;
import com.sxtvs.open.api.h5template.service.IH5OperateFormService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sxtvs.open.core.advice.BizException;
import com.sxtvs.open.core.sls.AliyunLogger;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2023-04-04
 */
@Service
public class H5OperateFormServiceImpl extends ServiceImpl<H5OperateFormMapper, H5OperateForm> implements IH5OperateFormService {

    @Resource
    private ObjectMapper objectMapper;

    @Resource
    private H5OperateFormDataServiceImpl h5OperateFormDataService;

    @Resource
    private AliyunLogger logger;

    public void store(H5Operate h5Operate){
        Snowflake snowflake = IdUtil.getSnowflake();

        try {

            List<Map<String, Object>> hoverList = objectMapper.readValue(h5Operate.getHoversList(), List.class);
            for (Map<String, Object> hover : hoverList){
                for (Map<String, Object> component : ((List<Map<String, Object>>)hover.get("components"))){
                    if("fromComponent".equals(component.get("type"))){
                        String formId = String.valueOf(snowflake.nextId());
                        for (Map<String, Object> form : ((List<Map<String, Object>>)component.get("child_list"))){

                            Map<String, Object> attr = (Map<String, Object>) form.get("attr");
                            if(!attr.containsKey("key")){
                                continue;
                            }
                            H5OperateForm h5OperateForm = new H5OperateForm();
                            h5OperateForm.setOperateId(h5Operate.getOperateId());
                            h5OperateForm.setColumnKey((String) attr.get("key"));
                            h5OperateForm.setLabel((String) attr.get("label"));
                            h5OperateForm.setFormId(formId);
                            save(h5OperateForm);
                        }

                        component.put("formId", formId);
                        component.put("operateId", h5Operate.getOperateId());
                    }
                }
            }
            h5Operate.setHoversList(objectMapper.writeValueAsString(hoverList));
        } catch (JsonProcessingException e) {
            throw new BizException("数据解析失败");
        }

    }

    public void update(H5Operate h5Operate){
        if(h5OperateFormDataService.lambdaQuery().eq(H5OperateFormData::getOperateId, h5Operate.getOperateId()).count() == 0 ){
            LambdaQueryWrapper<H5OperateForm> wrapper = Wrappers.lambdaQuery();
            remove(wrapper.eq(H5OperateForm::getOperateId, h5Operate.getOperateId()));
            store(h5Operate);
        }else {
            logger.debug("operate update", "活动页已产生用户数据，更改活动页无法修改表单");
        }
    }
}
