package com.sxtvs.open.api.user.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.sxtvs.open.api.user.dto.*;
import com.sxtvs.open.api.user.entity.User;
import com.sxtvs.open.api.user.service.impl.UserServiceImpl;
import com.sxtvs.open.core.auth.LoginRequired;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zyx
 * @since 2023-02-15
 */
@RestController
@RequestMapping("/user")
public class UserController {

    @Autowired
    private UserServiceImpl userService;

    @PostMapping("/login")
    public TokenDto login(@RequestBody @Valid LoginDto dto) {
        return userService.login(dto);
    }

    @RequestMapping("/info")
    @LoginRequired
    public User info() {
        return userService.getLoginUser();
    }

    @RequestMapping("/logout")
    @LoginRequired
    public void logout() {
        userService.logout();
    }

    @RequestMapping("pwd/reset")
    @LoginRequired
    public void reset(@RequestBody PwdResetDTO pwdResetDTO){
        userService.reset(pwdResetDTO);
    }

    @RequestMapping("list")
    @LoginRequired
    public Page<User> users(@RequestBody UserDTO userDTO){
        return userService.users(userDTO);
    }

    @RequestMapping("update")
    @LoginRequired
    public void update(@RequestBody UserDTO userDTO){
        userService.updateRole(userDTO);
    }

    @RequestMapping("store")
    @LoginRequired
    public void store(@RequestBody UserStoreDTO storeDTO){
        userService.createUser(storeDTO.getLoginName(), storeDTO.getLoginPassword(), storeDTO.getRoleId());
    }

}
