package com.sxtvs.open.core.auth;


import com.sxtvs.open.core.advice.BizException;
import com.sxtvs.open.core.conf.Constant;
import jakarta.servlet.http.Cookie;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.AsyncHandlerInterceptor;

import java.lang.reflect.Method;
import java.util.Arrays;

public class LoginInterceptor implements AsyncHandlerInterceptor {
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        if (!(handler instanceof HandlerMethod handlerMethod)) {
            return true;
        }
        HttpContextUtil.setRequestId();
        Method method = handlerMethod.getMethod();

        LoginRequired methodAnnotation = method.getAnnotation(LoginRequired.class);
        if (methodAnnotation == null) {
            return true;
        }

        // 优先取header
        var token = request.getHeader(Constant.TOKEN_HEADER_NAME);

        // 接下来取cookie
        var cookies = request.getCookies();
        if (StringUtils.isBlank(token) && cookies != null) {
            token = Arrays.stream(request.getCookies())
                    .filter(x -> x.getName().equals(Constant.TOKEN_COOKIE_NAME))
                    .map(Cookie::getValue)
                    .findFirst()
                    .orElse("");
        }

        if (!AESUtil.isOk(token)) {
            throw new BizException(40001, "没有登录");
        }
        HttpContextUtil.setToken(token);

        return true;
    }
}
